/*
Name:           fibonacci
Description:    This function calculates fibonacci numbers for endless iterations
Author:         Franklin van de Meent (https://frankl.in)
Source & docs:  https://github.com/fvdm/nodejs-fibonacci
Contact:        https://github.com/fvdm/nodejs-fibonacci/issues
License:        Unlicense (Public Domain, see UNLICENSE file)
                <https://github.com/fvdm/nodejs-fibonacci/raw/master/UNLICENSE>
*/

const bignum = require ('bn.js');
const { EventEmitter } = require ('events');

module.exports = new EventEmitter();
module.exports.doWhile = false;


/**
 * Start iteration
 *
 * @param   {number}  [limit=0]  Run fibonacci iterations
 * @return  {object}             result or limitResult
 */

function startIteration (limit) {
  let next = new bignum (1);
  let cur = new bignum (-1);
  let last = new bignum (0);
  let loop = new bignum (0);
  let start = new Date().getTime();
  let result = {};
  let limitResult = {};

  limit = limit && new bignum (limit);
  module.exports.doWhile = true;

  while (module.exports.doWhile) {
    // prev cur -> now last
    // prev next -> now cur
    last = cur;
    cur = next;
    next = cur.add (last);

    result.number = next.toString();
    result.length = next.toString().length;
    result.iterations = loop.toString();
    result.ms = new Date().getTime() - start;

    module.exports.emit ('result', result);

    // found the one
    if (limit && loop.eq (limit)) {
      module.exports.doWhile = false;
      module.exports.emit ('done', result);
    }

    // catch infinity
    if (next === 'Infinity') {
      limitResult = {
        reason: 'infinity',
        max_limit: Number.MAX_LIMIT.toString(),
        last_result: result,
        iterations: loop.toString(),
        intended: limit ? limit : null
      };

      module.exports.doWhile = false;
      module.exports.emit ('stop', limitResult);

      return limitResult;
    }

    // count
    loop = loop.add (new bignum (1));
  }

  return result;
}


/**
 * Stop iteration
 *
 * @return {void}
 */

function killIteration () {
  module.exports.doWhile = false;
}


// ready
module.exports.iterate = startIteration;
module.exports.kill = killIteration;
